use color_eyre::Result;
use figment::providers::{Format, Toml};
use figment::Figment;
use serde::Deserialize;
use std::collections::HashMap;
use color_eyre::eyre::WrapErr;
use regex::Regex;
use validator::Validate;

#[derive(Deserialize, Debug, Validate)]
pub struct Config {
    pub port: u16,
    pub sites_directory: String,
    pub sockets_directory: String,
    #[validate(nested)]
    pub scopes: HashMap<String, ConfigScope>
}

#[derive(Deserialize, Debug, Validate)]
pub struct ConfigScope {
    #[serde(with = "serde_regex")]
    pub domain_pattern: Regex,
    #[validate(length(equal = 100))]
    pub secret: String
}

pub fn load_config() -> Result<Config> {
    let config: Config = Figment::new()
        .merge(Toml::file("./config.toml"))
        .extract()
        .wrap_err("Failed to load the configuration.")?;

    Ok(config)
}